#include "DS3231.h"
// The following code is an implementation of DS3231 Real time clock and calender
//The   init_DS3231() must first be called before using the device
//
 //You can find the datasheet for this device at:   http://datasheets.maximintegrated.com/en/ds/DS3231.pdf
//Find attached code for setting the RTC
//
sbit LCD_RS at RC1_bit;
sbit LCD_EN at RC0_bit;
sbit LCD_D4 at RA4_bit;
sbit LCD_D5 at RA5_bit;
sbit LCD_D6 at RA6_bit;
sbit LCD_D7 at RA7_bit;


sbit LCD_RS_Direction at TRISC1_bit;
sbit LCD_EN_Direction at TRISC0_bit;
sbit LCD_D4_Direction at TRISA4_bit;
sbit LCD_D5_Direction at TRISA5_bit;
sbit LCD_D6_Direction at TRISA6_bit;
sbit LCD_D7_Direction at TRISA7_bit;

#define LCD_Light_TRIS  TRISC2_bit
#define LCD_Light       LATC2_bit

char txt_buffer[7], cursor = 1;
char rtc_hour, rtc_minute, rtc_second, rtc_ampm, rtc_dow, rtc_date, rtc_month, rtc_year;
char prev_minute = 0;

char set_hour, set_minute, set_ampm, set_day, set_dow, set_event_desc[8], set_recipient_phone[16];
char recipient_eeprom_address, recipient_phone[16], event_desc[8];
short rtc_hour_format;


char txt[3];
unsigned short kp, cnt, oldstate = 0;
char tmp, no_of_zeros;
char chr;// for buffering messages
unsigned int temp_conv;
char intro_msg[] = "REAL TIME CLOCK AND CALENDER WITH TEMP.";

char temp_str[8], disp_result,g;
void format_reminder_sms(char* sms_msg, char* sms_event_desc);


void initialize_LCD(){
     LCD_Light_TRIS = 0;
     LCD_Light = 1;//TURN ON LCD BACKLIGHT
     Lcd_Init();   //initialize LCD
     Lcd_Cmd(_LCD_CLEAR);                // Clear display
     Delay_ms(200);
     Lcd_Cmd(_LCD_CURSOR_OFF);
     Delay_ms(200);
     Lcd_Out(1,1, intro_msg);
     Delay_ms(1500);                // First row
     for(g=0; g<sizeof(intro_msg)-16; g++) {
        Lcd_Cmd(_LCD_SHIFT_LEFT);
        Delay_ms(700);
      }


}
void format_time(int value, char* result, char zeros){
//Zeros is the trailing zeros of the value eg:  6:08:09 instead of 6:8:9
        IntToStr(value, result);
        strcpy(result , Rtrim(result));
        strcpy(result , Ltrim(result));
        if(zeros>0)//pad with zero
        {
          *(result+1) = *(result);
          *(result)='0';
          *(result+2) = 0;

        }
}

char* day_of_week_int2str(char day_of_week){
  char lcd_DOW[4];
    switch(day_of_week){
        case  1:{
              strcpy(lcd_DOW, "SUN");
              break;
        }
        case  2:{
              strcpy(lcd_DOW, "MON");
              break;
        }
         case  3:{
              strcpy(lcd_DOW, "TUE");
              break;
        }
        case  4:{
              strcpy(lcd_DOW, "WED");
              break;
        }
        case  5:{
              strcpy(lcd_DOW, "THU");
              break;
        }
        case  6:{
              strcpy(lcd_DOW, "FRI");
              break;
        }
        case  7:{
              strcpy(lcd_DOW, "SUN");
              break;
        }
        return lcd_DOW;
    }
}
void main() {


     TRISA = 0;
    // TRISC = 0;


     //CONFIGURE INTERNAL OSC TO 32MHz
     IRCF2_bit = 1;
     IRCF1_bit = 1;
     IRCF0_bit = 1;

     Delay_ms(1000);// Wait for osc to stabilize
     ADCON1 = 0XFF;

     //INITIALIZE LCD
      initialize_LCD();

     //INITIALIZE RTC
     DS3231_init();
    setTime(7,51,0,AM,_12_hour_format); //set rtc time to 7:51:00 AM
    setDate(SUN,3,4,14);//set rtc date   //set date to SUNDAY, 29th April 2014



     while(1){


              Lcd_Cmd(_LCD_CLEAR);

              getDate(&rtc_dow, &rtc_date, &rtc_month, &rtc_year); //fetch date from RTC
              getTime(&rtc_hour, &rtc_minute, &rtc_second, &rtc_ampm,  _12_hour_format);  //fetch time from RTC

                //DISPLAY DATE

              Lcd_Out(1,1,day_of_week_int2str(rtc_dow));
              Lcd_Out_CP(" ");

              //DISPLAY TIME
              temp_conv = rtc_hour;
              format_time(temp_conv, txt_buffer,0);//do not prepend zero to hour
              Lcd_Out_CP(txt_buffer);

              Lcd_Out_CP(":");
              temp_conv = rtc_minute;  //show minute
              if(rtc_minute < 10)
                 no_of_zeros = 1;
              else
                 no_of_zeros = 0;
               format_time(temp_conv, txt_buffer, no_of_zeros);
              Lcd_Out_CP(txt_buffer);
              
              //UNCOMMENT THIS CODE TO DISPLAY SECONDS
               /*
              Lcd_Out_CP(":");     //show second
              temp_conv = rtc_second;
              if(rtc_second < 10)
                 no_of_zeros = 1;
              else
                 no_of_zeros = 0;
               format_time(temp_conv, txt_buffer, no_of_zeros);
               Lcd_Out_CP(txt_buffer);
                */
              
              //convert ampm to decimal
              if(rtc_ampm)
                  Lcd_Out_CP("PM");
               else
                 Lcd_Out_CP("AM");



              //THE FOLLOWING CODE DISPLAYS DATE DD//MM/YYYY
              temp_conv = rtc_date;  //show date
              format_time(temp_conv, txt_buffer,0);
              Lcd_Out(2,4,txt_buffer);//date

              Lcd_Chr_CP('/');

              temp_conv = rtc_month;  //show month
              format_time(temp_conv, txt_buffer,0);
              Lcd_Out_CP(txt_buffer);//display month

              Lcd_Chr_CP('/');


              temp_conv = rtc_year;  //show month
              format_time(temp_conv, txt_buffer,0);
              Lcd_Out_CP(txt_buffer);//display month


              Delay_ms(900);   //delay for 900ms then refresh screen


     }



}